/*
 * ******************************************************************************
 *  Copyright © 2016 Ruben Carmona Zurita. All rights reserved. The total or
 *  partial reproduction of this work it is forbidden by any means or process,
 *  including printing, reprography, microfilm, computer processing or any
 *  other system, as well as the distribution of copies by rental and lending,
 *  without the written permission of the author or the limits authorized by
 *  the law of intellectual property.
 *  ******************************************************************************
 */

package com.uoc.tfm.analyzerhandler.activities;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;

import com.uoc.tfm.analyzerhandler.R;
import com.uoc.tfm.analyzerhandler.common.AppGlobals;
import com.uoc.tfm.analyzerhandler.common.PushNotificationUtil;
import com.uoc.tfm.analyzerhandler.entities.UserData;
import com.uoc.tfm.analyzerhandler.util.SharePreferenceUtil;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

/**
 * A login screen that offers login via email/password.
 */
public class LoginActivity extends AppCompatActivity {

    private static final String TAG = LoginActivity.class.getSimpleName();


    /**
     * Keep track of the login task to ensure we can cancel it if requested.
     */
    private UserLoginTask mAuthTask = null;

    // UI references.
    private EditText etxtUsername;
    private EditText etxtPassw;
    private EditText etxtServerIP;
    private View mProgressView;
    private View mLoginFormView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        // Set up the login form.
        etxtUsername = (EditText) findViewById(R.id.etxtUsername);
        etxtPassw = (EditText) findViewById(R.id.etxtPassword);
        etxtServerIP = (EditText) findViewById(R.id.etxtServer);

        etxtServerIP.addTextChangedListener(new TextWatcher() {
            private String mPreviousText = "";

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                if (AppGlobals.PARTIAl_IP_ADDRESS.matcher(s).matches()) {
                    mPreviousText = s.toString();
                } else {
                    s.replace(0, s.length(), mPreviousText);
                }
            }
        });

        mLoginFormView = findViewById(R.id.login_form);
        mProgressView = findViewById(R.id.login_progress);

        etxtUsername.setText(SharePreferenceUtil.getStringValue(getApplicationContext(), "USERNAME"));
        etxtServerIP.setText(SharePreferenceUtil.getStringValue(getApplicationContext(), "SERVER_IP"));
        if (!TextUtils.isEmpty(etxtUsername.getText().toString())) {
            etxtPassw.requestFocus();
        }
    }


    /**
     * Attempts to sign in or register the account specified by the login form.
     * If there are form errors (invalid email, missing fields, etc.), the
     * errors are presented and no actual login attempt is made.
     */
    public void attemptLogin(View pView) {
        if (mAuthTask != null) {
            return;
        }

        // Reset errors.
        etxtUsername.setError(null);
        etxtPassw.setError(null);

        // Store values at the time of the login attempt.
        String username = etxtUsername.getText().toString();
        String password = etxtPassw.getText().toString();
        String serverIP = etxtServerIP.getText().toString();

        boolean cancel = false;
        View focusView = null;


        // Check for a valid username.
        if (TextUtils.isEmpty(username)) {
            etxtUsername.setError(getString(R.string.error_field_required));
            focusView = etxtUsername;
            cancel = true;
        }
        // Check for a valid password, if the user entered one.
        if (!cancel && !TextUtils.isEmpty(password) && !isPasswordValid(password)) {
            etxtPassw.setError(getString(R.string.error_invalid_password));
            focusView = etxtPassw;
            cancel = true;
        }


        // Check for a valid server address.
        if (!cancel && TextUtils.isEmpty(serverIP)) {
            etxtServerIP.setError(getString(R.string.error_field_required));
            focusView = etxtServerIP;
            cancel = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            focusView.requestFocus();
        } else {
            // Show a progress spinner, and kick off a background task to
            // perform the user login attempt.
            AppGlobals.SERVER_IP = serverIP;

            SharePreferenceUtil.setStringValue(getApplicationContext(), "USERNAME", username);
            SharePreferenceUtil.setStringValue(getApplicationContext(), "SERVER_IP", serverIP);

            showProgress(true);

            mAuthTask = new UserLoginTask(username, password);
            mAuthTask.execute();
        }
    }


    private boolean isPasswordValid(String password) {
        //TODO: Replace this with your own logic
        return password.length() >= 2;
    }

    /**
     * Shows the progress UI and hides the login form.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
    private void showProgress(final boolean show) {
        // On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
        // for very easy animations. If available, use these APIs to fade-in
        // the progress spinner.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
            int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

            mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
            mLoginFormView.animate().setDuration(shortAnimTime).alpha(
                    show ? 0 : 1).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
                }
            });

            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mProgressView.animate().setDuration(shortAnimTime).alpha(
                    show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
                }
            });
        } else {
            // The ViewPropertyAnimator APIs are not available, so simply show
            // and hide the relevant UI components.
            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
        }
    }

    /**
     * Represents an asynchronous login/registration task used to authenticate
     * the user.
     */
    public class UserLoginTask extends AsyncTask<Void, Void, ResultSet> {

        private String username;
        private String passw;

        public UserLoginTask(String pUsername, String pPassw) {
            username = pUsername;
            passw = pPassw;
        }

        @Override
        protected ResultSet doInBackground(Void... params) {
            try {
                ResultSet result = null;
                if (InetAddress.getByName(AppGlobals.SERVER_IP).isReachable(5000)) {
                    if (AppGlobals.conn == null || (AppGlobals.conn != null && AppGlobals.conn.isClosed())) {
                        Class.forName("com.mysql.jdbc.Driver");
                        DriverManager.setLoginTimeout(10);
                        AppGlobals.conn = DriverManager.getConnection("jdbc:mysql://" + AppGlobals.SERVER_IP + ":3306" + AppGlobals.DATABASE_NAME, "tfm_user", "tfm_user");
                    }
                    Statement stmt = AppGlobals.conn.createStatement();

                    String sqlStmt = "SELECT * FROM Users WHERE username='" + username + "' AND password='" + passw + "'";

                    result = stmt.executeQuery(sqlStmt);
                }
                return result;
            } catch (SQLException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(final ResultSet pRS) {
            mAuthTask = null;
            showProgress(false);
            try {
                if (pRS != null && pRS.next()) {
                    AppGlobals.userData = new UserData();
                    AppGlobals.userData.fromResultSet(pRS);
                    AppGlobals.UNIFIED_PUSH_URL = AppGlobals.UNIFIED_PUSH_URL.replaceAll("<server>", AppGlobals.SERVER_IP);
                    AppGlobals.UNIFIED_PUSH_PUBLIC_URL = AppGlobals.UNIFIED_PUSH_PUBLIC_URL.replaceAll("<server>", AppGlobals.SERVER_IP);
                    savePreferencesForFirstTime();
                    Intent intent = new Intent(LoginActivity.this, MainActivity.class);
                    startActivity(intent);
                } else {
                    etxtPassw.setText("");
                    etxtPassw.setError(getString(R.string.error_incorrect_password));
                    mAuthTask = null;
                    showProgress(false);
                }
            } catch (SQLException e) {
                e.printStackTrace();
            } finally {
                if (AppGlobals.conn != null) {
                    try {
                        AppGlobals.conn.close();
                        AppGlobals.conn = null;
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        @Override
        protected void onCancelled() {
            mAuthTask = null;
            showProgress(false);
        }

        private void savePreferencesForFirstTime() {
            String firstTime = SharePreferenceUtil.getStringValue(getApplicationContext(), "FIRST_TIME");

            if ("".equals(firstTime) || firstTime == null) {
                ArrayList<String> categoriesList = new ArrayList<>();
                categoriesList.add("A");
                categoriesList.add("W");
                categoriesList.add("I");
                categoriesList.add("C");
                PushNotificationUtil.registerToANotification(getApplicationContext(), categoriesList);
                SharePreferenceUtil.setStringValue(getApplicationContext(), "FIRST_TIME", AppGlobals.COMMON_NO);
            }
        }
    }
}