/*******************************************************************************
 * Copyright © 2016 Ruben Carmona Zurita.
 * All rights reserved. The total or partial reproduction of this work it is 
 * forbidden by any means or process, including printing, reprography, 
 * microfilm, computer processing or any other system, as well as the 
 * distribution of copies by rental and lending, without the written 
 * permission of the author or the limits authorized by the law of 
 * intellectual property.
 *******************************************************************************/
package com.tfm_uoc.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import com.tfm_uoc.common.UIUtils;
import com.tfm_uoc.database.TestsDAO;
import com.tfm_uoc.entities.AnalyzerData;
import com.tfm_uoc.entities.TestData;

/**
 * @project TFM - Analyzer Simulator
 * @author Rub
 * @filename WorklistTable.java
 */
public class TestConfigurationTable extends JTable
{

  private AnalyzerData analyzer;
  private WorklistTableModel wtm = new WorklistTableModel();
  private final String[] columnNames = { "Code", "Test Description", "To be done", "Remaining", "F. Warning" };

  private TestsDAO aTestsDAO = new TestsDAO();

  public TestConfigurationTable(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;
    this.setModel(wtm);
    this.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
    this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.setEnabled(true);
    this.getTableHeader().setReorderingAllowed(false);
    this.getTableHeader().setResizingAllowed(false);
    this.getTableHeader().setPreferredSize(new Dimension(-1, 30));
    this.setShowHorizontalLines(true);
    this.setShowVerticalLines(true);
    this.setBackground(UIUtils.nonSelectedBackground);
    this.setBorder(BorderFactory.createEmptyBorder());

    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(0), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(2), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(3), 100);
    UIUtils.setTableColumnFixedSize(this.getColumnModel().getColumn(4), 100);

    this.setRowHeight(35);

    for (int i = 0; i < columnNames.length; i++)
    {
      this.getColumnModel().getColumn(i).setCellRenderer(new MyTableCellRenderer());
    }

    DefaultCellEditor singleclick = new DefaultCellEditor(new NumericTextField());
    singleclick.setClickCountToStart(1);

    this.getColumnModel().getColumn(3).setCellEditor(singleclick);
    this.getColumnModel().getColumn(4).setCellEditor(singleclick);
  }

  public void updateAnalyzer(AnalyzerData pAnalyzer)
  {
    this.analyzer = pAnalyzer;
    wtm.fireTableDataChanged();
  }

  private class WorklistTableModel extends DefaultTableModel
  {
    public Class<?> getColumnClass(int column)
    {
      if (getValueAt(0, column) != null)
        return getValueAt(0, column).getClass();
      return null;
    }

    public boolean isCellEditable(int row, int column)
    {
      return column > 2;
    }

    public String getColumnName(int column)
    {
      return columnNames[column];
    }

    public int getRowCount()
    {
      if (analyzer != null)
      {
        return analyzer.getConfiguredTests().size();
      }
      else
      {
        return 0;
      }
    }

    public int getColumnCount()
    {
      return columnNames.length;
    }

    public Object getValueAt(int row, int column)
    {
      TestData aTest = analyzer.getConfiguredTests().get(row);

      switch (column)
      {
        case 0:
          return aTest.getCode();
        case 1:
          return aTest.getDescription();
        case 2:
          return aTest.getMinutes() + " min";
        case 3:
          return aTest.getFeasibility() + "";
        case 4:
          return aTest.getFeasibilityWarning() + "";
      }
      return "";
    }

    @Override
    public void setValueAt(Object aValue, int row, int column)
    {
      TestData aTestData = analyzer.getConfiguredTests().get(row);

      try
      {
        if (aTestData != null)
        {
          if (column == 3) // Feasibility in the Analyzer
          {
            aTestData.setFeasibility(new Integer(aValue.toString()));
            aTestsDAO.updateAnalyzerTestFeasibility(analyzer.getId(), aTestData.getId(), aTestData.getFeasibility());
          }
          else if (column == 4) // Feasibility Warning for the Test
          {
            aTestData.setFeasibilityWarning(new Integer(aValue.toString()));
            aTestsDAO.updateTestFeasibilityWarning(aTestData.getId(), aTestData.getFeasibilityWarning());
          }
        }
      }
      catch (ArrayIndexOutOfBoundsException _ex)
      {
      }

    }
  }

  private class MyTableCellRenderer implements TableCellRenderer
  {
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
    {
      JLabel aLabel;
      if (value instanceof String)
      {
        aLabel = new JLabel((String) value);
        aLabel.setOpaque(false);
        aLabel.setHorizontalTextPosition(column == 1 ? JLabel.LEFT : JLabel.CENTER);
        aLabel.setHorizontalAlignment(column == 1 ? JLabel.LEFT : JLabel.CENTER);
      }
      else
      {
        aLabel = (JLabel) value;
      }

      aLabel.setOpaque(true);
      aLabel.setBackground(row % 2 == 0 ? UIUtils.nonSelectedBackground : Color.WHITE);
      return aLabel;
    }
  }

  class NumericTextField extends JTextField
  {
    public NumericTextField()
    {
      this.setColumns(5);
    }

    @Override
    protected Document createDefaultModel()
    {
      return new NumericDocument();
    }

    private class NumericDocument extends PlainDocument
    {
      // The regular expression to match input against (zero or more digits)
      private final Pattern DIGITS = Pattern.compile("\\d*");

      @Override
      public void insertString(int offs, String str, AttributeSet a) throws BadLocationException
      {
        // Only insert the text if it matches the regular expression
        if (str != null && DIGITS.matcher(str).matches() && (getLength() + str.length()) <= 5)
          super.insertString(offs, str, a);
      }
    }
  }
}
